<?php
/*
Plugin Name: wp_mail Cyrillic
Version: 0.25
Plugin URI: http://mywordpress.ru/plugins/wp_mail_cyr
Description: Allows to receive e-mail messages in character sets different from the blog charset. Based on the original plugin by Anton Skorobogatov.
Author: Sergey Biryukov
Author URI: http://sergeybiryukov.ru/
*/

$utf8_cp1251_map = array(
"\xD0\x8E" => "\xA1",        // ў
"\xD1\x9E" => "\xA2",        // Ў
"\xD0\x81" => "\xA8",        // Ё
"\x04\x04" => "\xAA",        // Є
"\xD0\x87" => "\xAF",        // Ї
"\x04\x06" => "\xB2",        // ?
"\x04\x56" => "\xB3",        // ?
"\xD1\x91" => "\xB8",        // ё
"\x04\x54" => "\xBA",        // є
"\x04\x57" => "\xBF",        // ї
"\xD0\x90" => "\xC0",        // А русское
"\xD0\x91" => "\xC1",        // Б
"\xD0\x92" => "\xC2",        // В
"\xD0\x93" => "\xC3",        // и т.д.
"\xD0\x94" => "\xC4",
"\xD0\x95" => "\xC5",
"\xD0\x96" => "\xC6",
"\xD0\x97" => "\xC7",
"\xD0\x98" => "\xC8",
"\xD0\x99" => "\xC9",
"\xD0\x9A" => "\xCA",
"\xD0\x9B" => "\xCB",
"\xD0\x9C" => "\xCC",
"\xD0\x9D" => "\xCD",
"\xD0\x9E" => "\xCE",
"\xD0\x9F" => "\xCF",
"\xD0\xA0" => "\xD0",
"\xD0\xA1" => "\xD1",
"\xD0\xA2" => "\xD2",
"\xD0\xA3" => "\xD3",
"\xD0\xA4" => "\xD4",
"\xD0\xA5" => "\xD5",
"\xD0\xA6" => "\xD6",
"\xD0\xA7" => "\xD7",
"\xD0\xA8" => "\xD8",
"\xD0\xA9" => "\xD9",
"\xD0\xAA" => "\xDA",
"\xD0\xAB" => "\xDB",
"\xD0\xAC" => "\xDC",
"\xD0\xAD" => "\xDD",
"\xD0\xAE" => "\xDE",
"\xD0\xAF" => "\xDF",
"\xD0\xB0" => "\xE0",
"\xD0\xB1" => "\xE1",
"\xD0\xB2" => "\xE2",
"\xD0\xB3" => "\xE3",
"\xD0\xB4" => "\xE4",
"\xD0\xB5" => "\xE5",
"\xD0\xB6" => "\xE6",
"\xD0\xB7" => "\xE7",
"\xD0\xB8" => "\xE8",
"\xD0\xB9" => "\xE9",
"\xD0\xBA" => "\xEA",
"\xD0\xBB" => "\xEB",
"\xD0\xBC" => "\xEC",
"\xD0\xBD" => "\xED",
"\xD0\xBE" => "\xEE",
"\xD0\xBF" => "\xEF",
"\xD1\x80" => "\xF0",
"\xD1\x81" => "\xF1",
"\xD1\x82" => "\xF2",
"\xD1\x83" => "\xF3",
"\xD1\x84" => "\xF4",
"\xD1\x85" => "\xF5",
"\xD1\x86" => "\xF6",
"\xD1\x87" => "\xF7",
"\xD1\x88" => "\xF8",
"\xD1\x89" => "\xF9",
"\xD1\x8A" => "\xFA",
"\xD1\x8B" => "\xFB",
"\xD1\x8C" => "\xFC",
"\xD1\x8D" => "\xFD",        // э
"\xD1\x8E" => "\xFE",        // ю
"\xD1\x8F" => "\xFF",        // я
"\xC2\xAB" => "\x27",        // «
"\xC2\xBB" => "\x27",        // »
"\xC2\xB0" => "\xB0",        // °
"\xE2\x80\x9C" => "\x22",    // "
"\xE2\x80\x9D" => "\x22",    // "
"\xE2\x80\xA2" => "\x95",    // •
"\xE2\x80\xA6" => "\x2E",    // …
"\xE2\x80\x93" => "\x96",    // –
"\xE2\x80\x94" => "\x97",    // —
"\xE2\x84\x96" => "\xB9"     // №
);

if ( !function_exists('wp_mail_cyr_convert')) :
function wp_mail_cyr_convert($from, $to, $subject) {
	if ( !strcmp($from, $to) ) {
		return $subject;
	}
	global $utf8_cp1251_map;
	if ( function_exists('iconv') ) {
		switch ($to) {
			case 'windows-1251':
				$to_standard = 'CP1251';
				break;
			case 'koi8-r':
				$to_standard = 'KOI8-R';
				break;
		}
		return iconv($from, $to_standard, $subject);
	} elseif ( function_exists('mb_convert_encoding') ) {
		return mb_convert_encoding($subject, $to, $from);
	} elseif ( $to == 'windows-1251' && $from == 'UTF-8' ) {
		return strtr($subject, $utf8_cp1251_map);
	} elseif ( $to == 'koi8-r' && $from == 'UTF-8' ) {
		return convert_cyr_string(strtr($subject, $utf8_cp1251_map), 'w', 'k');
	}
}
endif;

// Generate MIME encoded-word for mail headers (RFC 2047)
if ( !function_exists('wp_mail_mime_encode') ) :
function wp_mail_mime_encode($str, $charset = 'ISO-8859-1') {
	if ( function_exists('iconv_mime_encode') ) {
		$preferences = array(
			"input-charset" => $charset,
			"output-charset" => $charset,
			"line-break-chars" => "\n"
		);
		$encoded_word = iconv_mime_encode('name', $str, $preferences);
		return substr($encoded_word, 6);
	} elseif ( function_exists('mb_encode_mimeheader') ) {
		mb_internal_encoding($charset);
		return mb_encode_mimeheader($str, $charset, 'B', "\n");
	} else {
		$encoded_word = '=?' . $charset . '?B?' . base64_encode($str) . '?=';
		return $encoded_word;
	}
}
endif;

if ( !function_exists('wp_mail') ) :
function wp_mail($to, $subject, $message, $headers = '') {
	$blogcharset = get_settings('blog_charset');
	$mailcharset = get_option('wp_mail_cyr_charset');
	if ( empty($mailcharset) ) {
		$mailcharset = $blogcharset;
	}
	if ( empty($headers) ) {
		$headers = "MIME-Version: 1.0\n" .
		$headers .= "From: " . get_settings('admin_email') . "\n";
		$headers .= "Content-Type: text/plain; charset=\"$mailcharset\"\n";
	}
	$to = wp_mail_cyr_convert($blogcharset, $mailcharset, $to);
	$subject = wp_mail_cyr_convert($blogcharset, $mailcharset, $subject);
	$subject = htmlspecialchars_decode($subject);
	$subject = wp_mail_mime_encode($subject, $mailcharset);
	$message = wp_mail_cyr_convert($blogcharset, $mailcharset, $message);
	$message = chunk_split(base64_encode($message), 76, "\n");
	$headers = str_replace($blogcharset, $mailcharset, $headers);
	$pattern = "/From: ([\"]*)([^\"]+)([\"]*) <(.+)>/i";
	if ( preg_match($pattern, $headers, $matches) ) {
		$from = wp_mail_cyr_convert($blogcharset, $mailcharset, $matches[2]);
		$from = wp_mail_mime_encode($from, $mailcharset);
		$replacement = "From: " . $from . " <\${4}>";
		$headers = preg_replace($pattern, $replacement, $headers);
	}
	$headers .= "Content-Transfer-Encoding: base64\n";
	$headers .= "Content-Disposition: inline";
	return @mail($to, $subject, $message, $headers);
}
endif;

if ( !function_exists('wp_mail_cyr_options_page') ) :
function wp_mail_cyr_options_page() {
?>
<div class="wrap">
	<h2><?php _e('Setup Charsets', 'mywordpress.ru'); ?></h2>
	<?php
	if ( $_POST['wp_mail_cyr_charset'] ) {
		update_option('wp_mail_cyr_charset', $_POST['wp_mail_cyr_charset']);
		echo '<div class="updated"><p>' . __('Options saved.') . '</p></div>';
	}
	?>
	<form method="post">
	<fieldset class="options">
		<legend><?php _e('Send e-mails in this charset:', 'mywordpress.ru'); ?></legend>
		<?php $charset = get_option('wp_mail_cyr_charset'); ?>
		<select name="wp_mail_cyr_charset">
			<option value="UTF-8"<?php if ( $charset == 'UTF-8' || empty($charset) ) { echo(' selected="selected"'); } ?>>UTF-8</option>
			<option value="windows-1251"<?php if ( $charset == 'windows-1251' ) { echo(' selected="selected"'); } ?>>Windows-1251</option>
			<option value="koi8-r"<?php if ( $charset == 'koi8-r' ) { echo(' selected="selected"'); } ?>>KOI8-R</option>
		</select>
		<input type="submit" value="<?php _e('Change', 'mywordpress.ru'); ?>" />
	</fieldset>
	</form>
</div>
<?
}
endif;

if ( !function_exists('wp_mail_cyr_add_menu') ) :
function wp_mail_cyr_add_menu() {
	add_options_page(__('Character Sets', 'mywordpress.ru'), __('Character Sets', 'mywordpress.ru'), 8, __FILE__, 'wp_mail_cyr_options_page');
}
endif;

if ( function_exists('load_plugin_textdomain') ) {
	load_plugin_textdomain('mywordpress.ru', 'wp-content/plugins/wp_mail_cyr');
}
add_action('admin_menu', 'wp_mail_cyr_add_menu');
?>
